---
-- Class with associative map and list for achievement IDs.
-- @type AchievementIdList
-- @field #table list
-- @field #table map
-- @field #number size
local AchievementIdList = {}

TITAN_ACHIEVEMENT_ID_LIST = AchievementIdList

AchievementIdList.__index = AchievementIdList

setmetatable(AchievementIdList, {
    __call = function (cls, ...)
        local self = setmetatable({}, AchievementIdList)
        self.list = {}
        self.map = {}
        self.size = 0
        return self
    end,
})

---
-- Check if achievement ID is in AchievementList.
--
-- @function [parent=#AchievementIdList] contains
-- @param self "this" reference
-- @param #number achievementID The ID to check.
-- @return #boolean true, if in list.
function AchievementIdList:contains(achievementID)
    if (self.map[achievementID] == true) then
        return true
    else
        return false
    end
end

---
-- Add an achievement ID.
--
-- @function [parent=#AchievementIdList] add
-- @param self "this" reference
-- @param #number achievementID The ID to add.
-- @return #nil void
function AchievementIdList:add(achievementID)
    if (not self:contains(achievementID)) then
        self.size = self.size + 1
        table.insert(self.list,achievementID)
        self.map[achievementID] = true
    end
end

---
-- Remove an achievement ID
--
-- @function [parent=#AchievementIdList] remove
-- @param self "this" reference
-- @param #number achievementID The ID to remove.
-- @return #nil void
function AchievementIdList:remove(achievementID)
    if (self:contains(achievementID)) then
        self.map[achievementID] = nil
        self.size = self.size - 1
        for pos, id in ipairs(self.list) do
            if (id == achievementID) then
                table.remove(self.list, pos)
                return
            end
        end
    end
end

---
-- Remove an achievement ID at given position
--
-- @function [parent=#AchievementIdList] removeAt
-- @param self "this" reference
-- @param #number pos The position in the list.
-- @return #nil void
function AchievementIdList:removeAt(pos)
    local value = self.list[pos]
    if (value ~= nil) then
        self.map[value] = nil
        self.size = self.size - 1
        table.remove(self.list,pos)
    end
end

---
-- Move an achievement up in the list
--
-- @function [parent=#AchievementIdList] moveUp
-- @param self "this" reference
-- @param #number pos The position to move up.
-- @return #nil void
function AchievementIdList:moveUp(pos)
    local temp = self.list[pos]
    self.list[pos] = self.list[pos-1]
    self.list[pos-1] = temp
end

---
-- Move an achievement down in the list
--
-- @function [parent=#AchievementIdList] moveDown
-- @param self "this" reference
-- @param #number pos The position to move down.
-- @return #nil void
function AchievementIdList:moveDown(pos)
    local temp = self.list[pos]
    self.list[pos] = self.list[pos+1]
    self.list[pos+1] = temp
end