----------------------------------------------------------------------
--  Local variables
----------------------------------------------------------------------

local _G = _G

local L = LibStub("AceLocale-3.0"):GetLocale("TitanAchievementProgress", true)

---
-- defined in own file, will be set on ADDON_LOADED event
local AchievementIdList

---
-- @type AchievementIdList
local achievementIdList

-- Required Titan variables
TITAN_ACHIEVEMENTPROGRESS_ID = "AchievementProgress";
TITAN_ACHIEVEMENTPROGRESS_VERSION = "0.0.2";
TITAN_ACHIEVEMENTPROGRESS_UPDATE = 1.0; -- Update every x seconds

---
-- Print debug message replacing formating characters
--
-- @callof #_G
-- @param #string msg The message to print.
-- @return #nil void
local function debugPrint(msg)
    DEFAULT_CHAT_FRAME:AddMessage(msg:gsub("|", "!"));
end

----------------------------------------------------------------------
--  TitanPanelAchievementProgressButton_OnLoad(self)
----------------------------------------------------------------------
function _G.TitanPanelAchievementProgressButton_OnLoad(self)
    --
    -- LOCAL REGISTRY --
    --
    self.registry = {
        id = TITAN_ACHIEVEMENTPROGRESS_ID,
        version = TITAN_ACHIEVEMENTPROGRESS_VERSION,
        menuText = L.TITAN_ACHIEVEMENTPROGRESS_TITLE,
        buttonTextFunction = "TitanPanelAchievementProgressButton_GetButtonText",
        tooltipTitle = L.TITAN_ACHIEVEMENTPROGRESS_TITLE,
        tooltipTextFunction = "TitanPanelAchievementProgressButton_GetTooltipText",
        iconWidth = 16,
        icon = "INTERFACE\\ICONS\\achievement_quests_completed_twilighthighlands",
        category = "Information",
        controlVariables = {
            ShowIcon = true,
            ShowLabelText = true,
            DisplayOnRightSide = false,
        },
        savedVariables = {
            ShowIcon = 1,
            ShowLabelText = 1,
        }
    };

    --
    -- EVENT CATCHING --
    --
    self:RegisterEvent("ADDON_LOADED");
end

----------------------------------------------------------------------
-- TitanPanelAchievementProgressButton_OnEvent(self, event, ...)
----------------------------------------------------------------------
function _G.TitanPanelAchievementProgressButton_OnEvent(self, event, ...)
    if (... ~= "TitanAchievementProgress") then
        return
    end

    if (event == "ADDON_LOADED") then
        AchievementIdList = TITAN_ACHIEVEMENT_ID_LIST
        if ((TitanAchievementProgressVars == nil)
            or (TitanAchievementProgressVars.version ~= TITAN_ACHIEVEMENTPROGRESS_VERSION)) then
            TitanAchievementProgressVars = {
                version = TITAN_ACHIEVEMENTPROGRESS_VERSION,
                achievementIdList = AchievementIdList()
            }
        else
            setmetatable(TitanAchievementProgressVars.achievementIdList, AchievementIdList)
        end
        achievementIdList = TitanAchievementProgressVars.achievementIdList
        self:UnregisterEvent("ADDON_LOADED");
    end
end

----------------------------------------------------------------------
-- TitanPanelAchievementProgressButton_OnEnter(self)
----------------------------------------------------------------------
function _G.TitanPanelAchievementProgressButton_OnEnter(self)
    -- Update Titan button label and tooltip
    TitanPanelButton_UpdateButton(TITAN_ACHIEVEMENTPROGRESS_ID);
    TitanPanelButton_UpdateTooltip(self);
end


----------------------------------------------------------------------
-- local updateAchivementIdList()
----------------------------------------------------------------------
local function updateAchivementIdList()
    for _, achievementID in ipairs({GetTrackedAchievements()}) do
        if (not achievementIdList:contains(achievementID)) then
            local numCriteria = GetAchievementNumCriteria(achievementID)
            if (numCriteria > 0) then
                achievementIdList:add(achievementID)
            end
        end
    end
    TitanPanelButton_UpdateButton(TITAN_ACHIEVEMENTPROGRESS_ID)
end

----------------------------------------------------------------------
-- TitanPanelAchievementProgressButton_OnUpdate(self, elapsed)
----------------------------------------------------------------------
function _G.TitanPanelAchievementProgressButton_OnUpdate(self, elapsed)
    self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed

    if (self.TimeSinceLastUpdate > TITAN_ACHIEVEMENTPROGRESS_UPDATE) then
        updateAchivementIdList()
        self.TimeSinceLastUpdate = 0
    end
end

----------------------------------------------------------------------
-- TitanPanelAchievementProgressButton_OnClick(self, button)
----------------------------------------------------------------------
function _G.TitanPanelAchievementProgressButton_OnClick(self, button)
    -- Detect mouse clicks
    if (button == "LeftButton") then

    elseif (button == "MiddleButton") then

    end
end

----------------------------------------------------------------------
-- local removeAchievement(pos)
----------------------------------------------------------------------
local function removeAchievement(pos)
    achievementIdList:removeAt(pos)
    TitanPanelRightClickMenu_Close()
    updateAchivementIdList()
end

----------------------------------------------------------------------
-- local moveUpAchievement(pos)
----------------------------------------------------------------------
local function moveUpAchievement(pos)
    achievementIdList:moveUp(pos)
    TitanPanelRightClickMenu_Close()
    updateAchivementIdList()
end

----------------------------------------------------------------------
-- local moveDownAchievement(pos)
----------------------------------------------------------------------
local function moveDownAchievement(pos)
    achievementIdList:moveDown(pos)
    TitanPanelRightClickMenu_Close()
    updateAchivementIdList()
end

----------------------------------------------------------------------
-- local addAchievementRemoveUpDown(pos)
----------------------------------------------------------------------
local function addAchievementRemoveUpDown(pos)
    -- not the first -> add "move up" button
    if (pos > 1) then
        local info = {
            hasArrow = false,
            keepShownOnClick = false,
            notCheckable = true,
            text = L.TITAN_ACHIEVEMENTPROGRESS_MOVE_UP,
            func = function()
                moveUpAchievement(pos)
            end,
        }
        UIDropDownMenu_AddButton(info, _G.UIDROPDOWNMENU_MENU_LEVEL);
    end

    -- "remove" button
    local info = {
        hasArrow = false,
        keepShownOnClick = false,
        notCheckable = true,
        text = L.TITAN_ACHIEVEMENTPROGRESS_REMOVE,
        func = function()
            removeAchievement(pos)
        end,
    }
    UIDropDownMenu_AddButton(info, _G.UIDROPDOWNMENU_MENU_LEVEL);

    -- not the last -> add "move down" button
    if (pos < achievementIdList.size) then
        local info = {
            hasArrow = false,
            keepShownOnClick = false,
            notCheckable = true,
            text = L.TITAN_ACHIEVEMENTPROGRESS_MOVE_DOWN,
            func = function()
                moveDownAchievement(pos)
            end,
        }
        UIDropDownMenu_AddButton(info, _G.UIDROPDOWNMENU_MENU_LEVEL);
    end
end

----------------------------------------------------------------------
-- local addAchievementMenu()
----------------------------------------------------------------------
local function addAchievementMenu()
    for pos, achievementID in ipairs(achievementIdList.list) do
        local idNumber, name, points, completed, month, day, year, description,
            flags, image, rewardText, isGuildAch = GetAchievementInfo(achievementID)
        local info = {
            value = pos,
            hasArrow = true,
            keepShownOnClick = false,
            notCheckable = true,
            text = format("|cff00FFFF%s|r", tostring(name)),
        }
        UIDropDownMenu_AddButton(info);
    end

    if (achievementIdList.size > 0) then
        TitanPanelRightClickMenu_AddSpacer()
    end
end

----------------------------------------------------------------------
-- TitanPanelRightClickMenu_PrepareAchievementProgressMenu()
----------------------------------------------------------------------
function _G.TitanPanelRightClickMenu_PrepareAchievementProgressMenu()
    if _G.UIDROPDOWNMENU_MENU_LEVEL == 1 then
        TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_ACHIEVEMENTPROGRESS_ID].menuText);
        TitanPanelRightClickMenu_AddSpacer();

        addAchievementMenu()

        TitanPanelRightClickMenu_AddToggleIcon(TITAN_ACHIEVEMENTPROGRESS_ID);
        TitanPanelRightClickMenu_AddToggleLabelText(TITAN_ACHIEVEMENTPROGRESS_ID)         
        TitanPanelRightClickMenu_AddSpacer();
        TitanPanelRightClickMenu_AddCommand(L.TITAN_ACHIEVEMENTPROGRESS_MENU_HIDE, TITAN_ACHIEVEMENTPROGRESS_ID, TITAN_PANEL_MENU_FUNC_HIDE);
    elseif _G.UIDROPDOWNMENU_MENU_LEVEL == 2 then
        local pos = _G.UIDROPDOWNMENU_MENU_VALUE;
        if (type(pos) == "number") then
            addAchievementRemoveUpDown(pos)
        end
    end
end

----------------------------------------------------------------------
-- TitanPanelAchievementProgressButton_GetButtonText(id)
----------------------------------------------------------------------
function _G.TitanPanelAchievementProgressButton_GetButtonText(id)
    if(TitanGetVar(TITAN_ACHIEVEMENTPROGRESS_ID, "ShowLabelText") ~= nil) then
        return format("%s |cffFF9045%d|r", L.TITAN_ACHIEVEMENTPROGRESS_BUTTON_LABEL, achievementIdList.size)
    else
        return ""
    end
end

----------------------------------------------------------------------
-- TitanPanelAchievementProgressButton_GetTooltipText()
----------------------------------------------------------------------
function _G.TitanPanelAchievementProgressButton_GetTooltipText()
    local text = "";

    for _, achievementID in ipairs(achievementIdList.list) do
        local idNumber, name, points, completed, month, day, year, description,
            flags, image, rewardText, isGuildAch = GetAchievementInfo(achievementID)
        local line = ""
        local numCriteria = GetAchievementNumCriteria(achievementID)
        local currentLineLenght = #line
        local allCompleted = true
                
        if (numCriteria > 0) then
            -- has criteria -> list them with quantity 
            for criteriaNum = 1, numCriteria do
                local criteriaString, criteriaType, completed, quantity, reqQuantity,
                    charName, flags, assetID, quantityString, criteriaID =
                        GetAchievementCriteriaInfo(achievementID, criteriaNum)
                if (not completed) then
                    allCompleted = false
                end
                local criteriaInfo
                if (completed) then
                    criteriaInfo = format("|cff666666%s|r", criteriaString)
                elseif (reqQuantity <= 1) then
                    criteriaInfo = criteriaString
                else
                    criteriaInfo = format("%s |cffFFFFFF%d/%d|r", criteriaString, quantity, reqQuantity)
                end
                if (currentLineLenght + #criteriaInfo > 200) then
                    line = line .. "\n    " .. criteriaInfo
                    currentLineLenght = 4 + #criteriaInfo
                else
                    if (criteriaNum > 1) then
                        line = line .. " | " .. criteriaInfo
                        currentLineLenght = currentLineLenght + 3 + #criteriaInfo
                    else
                        line = line .. criteriaInfo
                        currentLineLenght = currentLineLenght + #criteriaInfo
                    end
                end
            end
        else
            -- no criteria -> just the description
            line = line .. description
        end
                
        if (allCompleted) then
            -- all criteria completed -> gray out
            line = format("|cff006666%s|r: ", name) .. line
        else
            line = format("|cff00FFFF%s|r: ", name) .. line
        end
        text = text .. line .. "\n"
    end

    if (text ~= "") then
        text = text:sub(1,-2)
    end

    if (text == "") then
        text = "-";
    end
    return text;
end